package com.me.common.ecs;

import com.me.common.ecs.event.Event;
import com.me.common.ecs.event.RegisteredListener;
import com.me.common.ecs.event.EventHandler;
import com.me.common.ecs.event.Listener;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class ListenerRegistry {

    private final Map<Class<? extends Event>, List<RegisteredListener>> registeredListeners;

    protected ListenerRegistry() {
        this.registeredListeners = new HashMap<>();
    }

    protected void registerListener(Listener listener) {
        Method[] methods = listener.getClass().getMethods();

        for (Method method : methods) {
            EventHandler eh = method.getAnnotation(EventHandler.class);
            if (eh == null) {
                continue;
            }

            Class<?>[] parameters = method.getParameterTypes();
            if (parameters.length != 1 || !Event.class.isAssignableFrom(parameters[0])) {
                System.err.println("Attempting to register an EventHandler with an invalid method signature: " + method.toGenericString() + " in " + listener.getClass());
                continue;
            }

            Class<? extends Event> eventClass = parameters[0].asSubclass(Event.class);
            method.setAccessible(true);

            List<RegisteredListener> executors = registeredListeners.get(eventClass);
            if (executors == null) {
                executors = new ArrayList<>();
                registeredListeners.put(eventClass, executors);
            }

            executors.add(new RegisteredListener(listener, method, eventClass, eh));
        }

    }

    protected void callEvent(Event event) {
        List<RegisteredListener> listeners = registeredListeners.get(event.getClass());
        if (listeners == null) {
            return;
        }

        for (RegisteredListener listener : listeners) {
            if (listener.isIgnoringCancelled() && event.isCancelled()) {
                continue;
            }

            listener.execute(event);
        }
    }

}
